@@ -174,7 +174,7 @@ class BaseAutoML(BaseEstimator, ABC):
             if os.path.exists(best_model_fname):
                 best_model_name = None
                 with open(best_model_fname, "r") as fin:
-                    best_model_name = fin.read()
+                    best_model_name = fin.read().rstrip()
                 self._best_model = models_map[best_model_name]
             else:
                 self._best_model = None
@@ -190,7 +190,7 @@ class BaseAutoML(BaseEstimator, ABC):
         except Exception as e:
             raise AutoMLException(f"Cannot load AutoML directory. {str(e)}")
 
-    def get_leaderboard(self):
+    def get_leaderboard(self, filter_random_feature=False):
         ldb = {
             "name": [],
             "model_type": [],
@@ -199,6 +199,9 @@ class BaseAutoML(BaseEstimator, ABC):
             "train_time": [],
         }
         for m in self._models:
+            # filter model with random feature
+            if filter_random_feature and "RandomFeature" in m.get_name():
+                continue
             ldb["name"] += [m.get_name()]
             ldb["model_type"] += [m.get_type()]
             ldb["metric_type"] += [self._eval_metric]
@@ -367,7 +370,7 @@ class BaseAutoML(BaseEstimator, ABC):
         if self._stacked_models is not None:
             return
 
-        ldb = self.get_leaderboard()
+        ldb = self.get_leaderboard(filter_random_feature=True)
         ldb = ldb.sort_values(by="metric_value", ascending=True)
 
         models_map = {m.get_name(): m for m in self._models if not m._is_stacked}
@@ -871,7 +874,11 @@ class BaseAutoML(BaseEstimator, ABC):
                     generated_params = self._all_params[step]
                 else:
                     generated_params = tuner.generate_params(
-                        step, self._models, self._results_path, self._stacked_models
+                        step,
+                        self._models,
+                        self._results_path,
+                        self._stacked_models,
+                        self._total_time_limit,
                     )
 
                 if generated_params is None or not generated_params:
@@ -1466,6 +1473,7 @@ class BaseAutoML(BaseEstimator, ABC):
             "rmse",
             "mse",
             "mae",
+            "r2",
         ]:
             raise ValueError(
                 f"Metric {self.eval_metric} is not allowed in ML task: {self._get_ml_task()}. \
